# Generated by Django 2.2.2 on 2019-09-04 17:15

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='PostalCode',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(blank=True, max_length=180, null=True)),
                ('latitude', models.DecimalField(blank=True, decimal_places=16, max_digits=22, null=True)),
                ('longitude', models.DecimalField(blank=True, decimal_places=16, max_digits=22, null=True)),
                ('country_code', models.CharField(blank=True, max_length=2, null=True)),
                ('postal_code', models.CharField(blank=True, max_length=20, null=True)),
                ('place_name', models.CharField(blank=True, max_length=180, null=True)),
                ('admin_name1', models.CharField(blank=True, max_length=100, null=True)),
                ('admin_code1', models.CharField(blank=True, max_length=20, null=True)),
                ('admin_name2', models.CharField(blank=True, max_length=100, null=True)),
                ('admin_code2', models.CharField(blank=True, max_length=20, null=True)),
                ('admin_name3', models.CharField(blank=True, max_length=100, null=True)),
                ('admin_code3', models.CharField(blank=True, max_length=20, null=True)),
                ('accuracy', models.IntegerField(choices=[(1, 'Estimated'), (4, 'Geoname ID'), (6, 'Centroid of addresses or shape')], null=True)),
                ('updated', models.DateTimeField(auto_now_add=True, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



from __future__ import unicode_literals

import datetime
import logging
import math
import os
import zipfile
from urllib.request import urlretrieve

from django.conf import settings
from django.core.management.base import BaseCommand

from geo_ez.utility_functions import import_postal_codes_csv

logger = logging.getLogger(__name__)

settings.DEBUG = False

countries = ["US", "AS", "GU", "MP", "PR", "VI", "AZ"]

insert_threshold = getattr(settings, "INSERT_THRESHOLD", 10000)
data_dir = getattr(settings, "DATA_DIR", os.path.join(settings.MEDIA_ROOT, "DATA"))


class Command(BaseCommand):
    help = "Import Postal Codes from GeoNames."
    verbosity = 0
    current_file = None
    log_file_name = None
    log_file = False

    init_time = None

    def _log_message(self, message):
        log_message = "%s: %s\n" % (datetime.datetime.now().isoformat()[0:19], message)

        logger.info(message)

        if self.verbosity > 0:
            self.stdout.write(log_message)

    def _timer(self):
        if not self.init_time:
            self.init_time = datetime.datetime.now()
            self._log_message("Command initiated.")
        else:
            self._log_message("Command completed.")

            complete_time = datetime.datetime.now()
            command_total_seconds = (complete_time - self.init_time).total_seconds()
            command_minutes = math.floor(command_total_seconds / 60)
            command_seconds = command_total_seconds - (command_minutes * 60)

            self._log_message("Command took %i minutes and %i seconds to run." % (command_minutes, command_seconds))

    def handle(self, *args, **options):
        self.verbosity = int(options["verbosity"])

        self._timer()

        zip_file_path = os.path.join(data_dir, "geonames")

        for country in countries:
            print("Processing: %s" % country)
            zip_file = os.path.join(zip_file_path, "%s.zip" % country)

            if not os.path.exists(zip_file_path):
                os.makedirs(zip_file_path)

            if os.path.exists(zip_file):
                os.remove(zip_file)

            urlretrieve("http://download.geonames.org/export/zip/%s.zip" % country, zip_file)

            zip_ref = zipfile.ZipFile(zip_file, "r")
            zip_ref.extractall(zip_file_path)
            zip_ref.close()
            os.remove(zip_file)

            data_file_path = os.path.join(zip_file_path, "%s.txt" % country)
            import_postal_codes_csv(data_file_path)

            os.remove(data_file_path)

        self._timer()



import json

from django.http import HttpResponse

from geo_ez.utility_functions import zip_codes_in_radius


def get_zipcodes_in_radius(request, **kwargs):
    """
    Finds ZipCodes within a radius of the specified Zip Code
    :param request:
    :return:
    """
    response = HttpResponse(content_type='application/json')
    response['Cache-Control'] = 'no-cache'

    response.write(json.dumps(dict(zip_codes=zip_codes_in_radius(**kwargs))))

    return response



# -*- coding: utf-8 -*-
from django.contrib import admin

from .models import PostalCode


@admin.register(PostalCode)
class PostalCodeAdmin(admin.ModelAdmin):
    list_display = ("country_code", "postal_code", "place_name", "admin_name1")
    list_filter = ("admin_name1",)
    search_fields = ("place_name", "postal_code", "admin_name1", "admin_code1", "admin_name2")



import csv
import datetime
import math

import bleach
from django.db.models.expressions import RawSQL
from django.utils.timezone import make_aware

from geo_ez.models import PostalCode
from geo_ez.us_census_class import USCensus
from geo_ez.usps_class import USPS


def geocode(address_dict, **kwargs):
    address_normalized = kwargs.get("normalized", False)

    search_address = dict(
        address1=address_dict.get("address1"),
        address2=address_dict.get("address2"),
        city=address_dict.get("city"),
        state=address_dict.get("state"),
        zip_code=address_dict.get("zip_code"),
    )

    if not address_normalized:
        # First, we normalize the address with the US Postal Service
        ps = USPS()
        valid_address = ps.address(**search_address)
    else:
        valid_address = search_address

    usc = USCensus()

    return usc.geocode(query=valid_address)


def miles_to_km(miles):
    return miles * 1.60934


def km_to_miles(km):
    return km * 0.621371


def deg2rad(deg):
    return deg * (math.pi / 180)


def get_distance(lat1, lon1, lat2, lon2, **kwargs):
    use_miles = kwargs.get("use_miles", True)

    lat1 = float(lat1)
    lat2 = float(lat2)
    lon1 = float(lon1)
    lon2 = float(lon2)

    radius = 6371  # Radius of the earth in km

    d_lat = deg2rad(lat2 - lat1)  # deg2rad below
    d_lon = deg2rad(lon2 - lon1)

    a = math.sin(d_lat / 2) * math.sin(d_lat / 2) + math.cos(deg2rad(lat1)) * math.cos(deg2rad(lat2)) * math.sin(
        d_lon / 2
    ) * math.sin(d_lon / 2)
    c = 2 * math.atan2(math.sqrt(a), math.sqrt(1 - a))
    distance = radius * c  # Distance in km

    if not use_miles:
        return_value = distance
    else:
        return_value = km_to_miles(distance)

    return return_value


def get_postal_code_by_coords(latitude, longitude):
    """
    Finds the nearest postal code to the provided coordinates.

    :param latitude:
    :param longitude:
    :return:
    """
    retn = None
    result_count = 0
    radius = 1
    while result_count == 0:
        postal_codes = postal_codes_within_radius(latitude, longitude, radius=radius)
        result_count = len(postal_codes)
        if result_count > 0:
            retn = postal_codes[0]
            break
        radius = radius + 1

    return retn


def points_within_radius(gismodel, latitude, longitude, **kwargs):
    radius = kwargs.get("radius", False)
    use_miles = kwargs.get("use_miles", True)

    if radius:
        radius = float(radius)

    distance_unit = float(3959 if use_miles else 6371)

    # Great circle distance formula
    gcd_formula = (
        "%s * acos(least(greatest(cos(radians(%s)) * cos(radians(latitude)) * cos(radians(longitude) - "
        "radians(%s)) + sin(radians(%s)) * sin(radians(latitude)), -1), 1))"
    )
    distance_raw_sql = RawSQL(gcd_formula, (distance_unit, latitude, longitude, latitude))
    qs = gismodel.objects.all().annotate(distance=distance_raw_sql).order_by("distance")

    if radius:
        qs = qs.filter(distance__lt=radius)

    return qs


def postal_codes_within_radius(latitude, longitude, **kwargs):
    return points_within_radius(PostalCode, latitude, longitude, **kwargs)


def import_postal_codes_csv(data_file_path, **kwargs):
    delimiter = kwargs.get("delimiter", "\t")

    data_file = open(data_file_path, "rU", encoding="utf8")

    rows = csv.reader(data_file, delimiter=delimiter)

    insert_list = []
    for row in rows:
        if len(row) > 0 and row[11]:
            try:
                postal_code = PostalCode.objects.get(postal_code=row[1], name=row[2], place_name=row[2])

            except PostalCode.DoesNotExist:
                insert_list.append(
                    PostalCode(
                        country_code=row[0],
                        postal_code=row[1],
                        name=row[2],
                        place_name=row[2],
                        admin_name1=row[3],
                        admin_code1=row[4],
                        admin_name2=row[5],
                        admin_code2=row[6],
                        admin_name3=row[7],
                        admin_code3=row[8],
                        latitude=row[9],
                        longitude=row[10],
                        accuracy=row[11],
                        updated=make_aware(datetime.datetime.now()),
                    )
                )

            else:
                postal_code.country_code = row[0]
                postal_code.postal_code = row[1]
                postal_code.name = row[2]
                postal_code.place_name = row[2]
                postal_code.admin_name1 = row[3]
                postal_code.admin_code1 = row[4]
                postal_code.admin_name2 = row[5]
                postal_code.admin_code2 = row[6]
                postal_code.admin_name3 = row[7]
                postal_code.admin_code3 = row[8]
                postal_code.latitude = row[9]
                postal_code.longitude = row[10]
                postal_code.accuracy = row[11]
                postal_code.updated = make_aware(datetime.datetime.now())
                postal_code.save()

    data_file.close()

    PostalCode.objects.bulk_create(insert_list)


def zip_codes_in_radius(**kwargs):
    zip_code = kwargs.get("zip_code", None)
    radius = kwargs.get("radius", False)
    distance_units = bleach.clean(kwargs.get("distance_units", "miles"))

    if distance_units.lower() in ["mi", "miles", "imperial", "empirical", "us", "united states", "usa"]:
        use_miles = True

    starting_zip_code = PostalCode.objects.get(postal_code=zip_code)

    zipcodes_in_radius = points_within_radius(
        PostalCode, starting_zip_code.latitude, starting_zip_code.longitude, radius=radius, use_miles=True
    )

    zip_codes = []
    for zip_code in zipcodes_in_radius:
        zip_codes.append(
            {
                "zip_code": zip_code.postal_code,
                "distance": round(zip_code.distance, 3),
                "city": zip_code.place_name,
                "state": zip_code.admin_code1,
            }
        )

    return zip_codes



# Generated by Django 2.2.2 on 2019-09-04 18:36

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('gis', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='StreetAddress',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(blank=True, max_length=180, null=True)),
                ('latitude', models.DecimalField(blank=True, decimal_places=16, max_digits=22, null=True)),
                ('longitude', models.DecimalField(blank=True, decimal_places=16, max_digits=22, null=True)),
                ('address1', models.CharField(blank=True, max_length=255, null=True)),
                ('address2', models.CharField(blank=True, max_length=255, null=True)),
                ('city', models.CharField(blank=True, max_length=180, null=True)),
                ('state', models.CharField(blank=True, max_length=20, null=True)),
                ('zip_code', models.CharField(blank=True, max_length=20, null=True)),
                ('plus_four', models.CharField(blank=True, max_length=20, null=True)),
                ('validated', models.BooleanField(default=False)),
                ('postal_code', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.DO_NOTHING, to='gis.PostalCode')),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



from __future__ import unicode_literals

import datetime
import logging
import math

from django.conf import settings
from django.core.management.base import BaseCommand

from geo_ez.models import PostalCode
from geo_ez.utility_functions import points_within_radius

logger = logging.getLogger(__name__)

settings.DEBUG = False


class Command(BaseCommand):
    help = "Import Postal Codes from GeoNames."
    verbosity = 0
    current_file = None
    log_file_name = None
    log_file = False

    init_time = None
    existing_drug_list = []
    drug_insert_list = []

    # def add_arguments(self, parser):
    #     parser.add_argument("address", type=str)

    def _log_message(self, message):
        log_message = "%s: %s\n" % (datetime.datetime.now().isoformat()[0:19], message)

        logger.info(message)

        if self.verbosity > 0:
            self.stdout.write(log_message)

    def _timer(self):
        if not self.init_time:
            self.init_time = datetime.datetime.now()
            self._log_message("Command initiated.")
        else:
            self._log_message("Command completed.")

            complete_time = datetime.datetime.now()
            command_total_seconds = (complete_time - self.init_time).total_seconds()
            command_minutes = math.floor(command_total_seconds / 60)
            command_seconds = command_total_seconds - (command_minutes * 60)

            self._log_message("Command took %i minutes and %i seconds to run." % (command_minutes, command_seconds))

    def handle(self, *args, **options):
        self.verbosity = int(options["verbosity"])

        self._timer()

        home_location = PostalCode.objects.get(postal_code="13212")
        # remote_location = PostalCode.objects.get(postal_code='13057')
        # zip_codes = postal_codes_within_radius(home_location.latitude, home_location.longitude, radius=5)
        zip_codes = points_within_radius(PostalCode, home_location.latitude, home_location.longitude, radius=5)

        for zip_code in zip_codes:
            print(zip_code.place_name, zip_code.postal_code, zip_code.distance)

        self._timer()



from django.urls import re_path

from geo_ez.views import get_zipcodes_in_radius

urlpatterns = [
    re_path(
        r"^zip-code/(?P<zip_code>\d+)/(?P<radius>[0-9\.]+)/(?P<distance_units>[a-zA-Z]+)/$",
        get_zipcodes_in_radius,
        name="get_zipcodes_in_radius",
    )
]
